local M = {}

local running = true

local vehicle = nil
local speeder = nil
local scenario = nil

local checkForPlayerCrash = false

local maxEnemyDamage = 55000 --no unit
local totalTime = 120 --seconds
local lastCountdownMsg = 11
local lastSeconds = false

local lastDamageMsg = 0
local damageMsgThreshold = 1000
local playerMinimumSpeed = 0.5 --m/s
local playerCrashTimeout = 0 --s
local playerCrashMaxTimeout = 10 --s

local function reset()
    checkForPlayerCrash = false
    playerCrashTimeout = 0
    running = false
    lastSeconds = false
end

local function onRaceStart()
    reset()   
    
    vehicle = scenetree.findObject('scenario_player0')
    speeder = scenetree.findObject('speeder')
    
    vehicle:queueLuaCommand('mapmgr.enableTracking("playerCar")')
    speeder:queueLuaCommand('ai.setState({mode="flee"})')    
    speeder:queueLuaCommand('mapmgr.enableTracking("speeder")')
    guihooks.trigger('ScenarioFlashMessage', {{'Destroy the Covet!', 2}} )
    running = true
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success()    
    local finalTime = scenario.timer
    local minutes = math.floor(finalTime / 60);
    local seconds = finalTime - (minutes * 60);
    local timeStr = ''
    if minutes > 0 then
        timeStr = string.format("%02.0f:%05.2f", minutes, seconds)
    else
        timeStr = string.format("%0.2f", seconds) .. 's'
    end
    local result = {msg = 'Well done, your time: '..timeStr}
    scenarios.finish(result)
    reset()
end

local function onRaceTick(raceTickTime)  
    if not running then
        return
    end
    
    local playerData = map.objects.playerCar
    local speederData = map.objects.speeder
    
    if not playerData or not speederData then
        return
    end
        
    local playerSpeed = playerData.vel:length()
    
    --Check if the player has passed the minimum speed before start watching his speed
    if not checkForPlayerCrash and playerSpeed > playerMinimumSpeed then
        checkForPlayerCrash = true
    end
    
    --Count down the player when he's below the minimum speed
    if checkForPlayerCrash and (playerSpeed < playerMinimumSpeed) and not lastSeconds then 
        playerCrashTimeout = playerCrashTimeout + raceTickTime 
        if playerCrashTimeout % 1 == 0 then
            local countdown = playerCrashMaxTimeout - math.modf(playerCrashTimeout)
            if countdown > 0 then
                guihooks.trigger('ScenarioFlashMessage', {{countdown..'...', 0.5}} )
            end
        end
    else
        playerCrashTimeout = 0
    end
    
    local damage = speederData.damage
    if damage - lastDamageMsg > damageMsgThreshold and damage < maxEnemyDamage then
        local progress = math.floor(damage / maxEnemyDamage * 100)
        guihooks.trigger('ScenarioFlashMessage', {{progress..'% damage', 1}} )
        lastDamageMsg = damage
    end
    
    --Count down the time left when below 10 seconds
    local timeLeft = totalTime - scenario.timer
    if timeLeft < 11 then 
        lastSeconds = true
        local countdown = math.floor(timeLeft) + 1
        if countdown > 0 and lastCountdownMsg > countdown then
            guihooks.trigger('ScenarioFlashMessage', {{countdown..'...', 0.8}} )
            lastCountdownMsg = countdown
        end
    end
        
    if scenario.timer >= totalTime then
        fail('You ran out of time')
    end
    
    if playerCrashTimeout >= playerCrashMaxTimeout then -- wait x seconds
        fail('You got stuck')
    end
    
    if speederData.damage >= maxEnemyDamage then
        success()
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M